package cn.estsh.i3plus.core.apiservice.controller.busi;

import cn.estsh.i3plus.core.api.iservice.busi.*;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.ExcelTool;
import cn.estsh.i3plus.platform.common.tool.StringTool;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.platform.common.util.FileContentTypeTool;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.model.platform.UserDetailModel;
import cn.estsh.i3plus.pojo.model.platform.UserExportModel;
import cn.estsh.i3plus.pojo.platform.bean.*;
import cn.estsh.impp.framework.base.controller.CoreBaseController;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import cn.estsh.impp.framework.boot.util.LocaleUtils;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.io.FileUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.persistence.EntityManager;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @Description : 用户信息 服务
 * @Reference :
 * @Author : wei.peng
 * @Date : 2018-11-08 19:57
 * @Modify :
 **/
@RestController
@Api(tags = "系统用户服务")
@RequestMapping(PlatformConstWords.BASE_URL + "/sys-user-info")
public class SysUserInfoController extends CoreBaseController {

    public static final Logger LOGGER = LoggerFactory.getLogger(SysUserInfoController.class);

    @Autowired
    private ILicenseClickService licenseClickService;

    @Autowired
    private ISysUserInfoService sysUserInfoService;

    @Autowired
    private ISysDepartmentService departmentService;

    @Autowired
    private ISysPositionService positionService;

    @Autowired
    private EntityManager entityManager;

    @Autowired
    private ICoreMemTreeService memTreeService;

    @Autowired
    private IPersonnelService personnelService;

    @Autowired
    private ISysUserService sysUserService;

    @Resource(name = CommonConstWords.IMPP_REDIS_RES)
    private ImppRedis redisRes;

    /**
     * 添加用户信息
     *
     * @param userInfo 用户信息
     * @return 处理结果
     */
    @PostMapping(value = "/insert")
    @ApiOperation(value = "添加用户信息", notes = "返回内容添加用户信息")
    public ResultBean insertSysUserInfo(SysUserInfo userInfo) {
        try {
            startMultiService();
            licenseClickService.checkLicenseNumberUser();

            //登陆用户
            SessionUser user = AuthUtil.getSessionUser();

            checkSysUserInfo(userInfo, false);

            ConvertBean.modelInitialize(userInfo, user);
            userInfo.setUserInfoStatus(CommonEnumUtil.USER_STATUS.ENABLE.getValue());
            userInfo.setUserLoginNum(0);

            sysUserInfoService.checkSysUserInfoOnly(userInfo);
            SysUserInfo info = sysUserInfoService.insertSysUserInfo(userInfo);
            sysUserInfoService.refreshRefUserOrganize(info.getId(), userInfo.getDepartment().getOrganizeId(), user.getUserName());
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultObject(info);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 根据用户信息id修改信息
     *
     * @param userInfo 用户信息
     * @return 处理结果
     */
    @PutMapping(value = "/update")
    @ApiOperation(value = "修改用户信息", notes = "根据用户信息id修改信息")
    public ResultBean updateSysUserInfoById(SysUserInfo userInfo) {
        try {
            startMultiService();

            //登陆用户
            SessionUser user = AuthUtil.getSessionUser();

            checkSysUserInfo(userInfo, true);

            //修改初始化
            ConvertBean.modelUpdate(userInfo, user);
            sysUserInfoService.checkSysUserInfoOnly(userInfo);

            sysUserInfoService.updateSysUserInfo(userInfo);

            sysUserInfoService.refreshRefUserOrganize(userInfo.getId(), userInfo.getDepartment().getOrganizeId(), user.getUserName());

            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 删除用户信息
     *
     * @param id 用户信息id
     * @return 处理结果
     */
    @DeleteMapping(value = "/delete/{id}")
    @ApiOperation(value = "删除用户信息", notes = "根据用户信息id删除信息")
    public ResultBean deleteSysUserInfo(@PathVariable("id") String id) {
        try {
            //条件验证
            ValidatorBean.checkNotNull(id, "id为空");

            sysUserInfoService.deleteSysUserInfoById(Long.parseLong(id));
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 查询所有用户信息
     *
     * @return 处理结果
     */
    @GetMapping(value = "/list")
    @ApiOperation(value = "查询所有用户信息", notes = "查询所有用户信息")
    public ResultBean findSysUserInfoAll() {
        try {
            List<SysUserInfo> SysUserInfoList = sysUserInfoService.listSysUserInfo();
            return ResultBean.success("操作成功").setResultList(SysUserInfoList)
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 查询用户信息
     *
     * @param userInfo 查询条件
     * @param pager    分页信息
     * @return 处理结果
     */
    @GetMapping(value = "/query")
    @ApiOperation(value = "查询用户信息", notes = "复杂查询用户信息带分页")
    public ResultBean querySysUserInfo(SysUserInfo userInfo, Pager pager) {
        try {
            ListPager<SysUserInfo> result = sysUserInfoService.querySysUserInfo(userInfo, packUserInfo(userInfo), pager);
            return ResultBean.success("操作成功").setListPager(result).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 根据id查询用户信息
     *
     * @param id 用户信息id
     * @return 处理结果
     */
    @GetMapping(value = "/get/{id}")
    @ApiOperation(value = "根据id查询用户信息", notes = "根据id用户信息")
    public ResultBean getSysUserInfoById(@PathVariable("id") String id) {
        try {
            ValidatorBean.checkIsNumber(id, "id 不能为空");

            SysUserInfo userInfo = sysUserInfoService.getSysUserInfoById(Long.parseLong(id));
            ValidatorBean.beginValid(userInfo).notNull("id", userInfo.getId());

            return ResultBean.success("操作成功").setResultObject(userInfo).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 根据id批量删除
     *
     * @param ids 用户id数组
     * @return 处理结果
     */
    @DeleteMapping(value = "/batch-delete")
    @ApiOperation(value = "根据id批量删除", notes = "根据id批量删除")
    public ResultBean deleteSysUserInfoByIds(String[] ids) {
        try {
            ids = ConvertBean.modelSafeArrayNumber(ids, true);
            if (ids.length == 0) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("请选择需要删除的用户信息。")
                        .build();
            }
            List<SysUserInfo> userInfoList = personnelService.findSysUserInfoByIds(StringTool.getArrayLong(ids));
            if (ObjectUtils.isEmpty(userInfoList)) {
                return ResultBean.fail("数据不存在").setCode(ResourceEnumUtil.MESSAGE.EMPTY.getCode());
            }
            Long[] userinfoIds = new Long[userInfoList.size()],
                    userIds = new Long[userInfoList.size()];
            for (int i = 0; i < userInfoList.size(); i++) {
                userinfoIds[i] = userInfoList.get(i).getId();
                userIds[i] = userInfoList.get(i).getUserId();
            }
            personnelService.deleteSysUserInfo(userinfoIds, getSessionUser().getUserName());
            personnelService.deleteSysUser(userIds, getSessionUser().getUserName());
            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 查询角色的所有角色权限关系
     *
     * @param userId 用户id
     * @return 处理结果
     */
    @GetMapping(value = "/find-ref-department/{userId}")
    @ApiOperation(value = "查询角色的所有角色权限关系", notes = "查询角色的所有角色权限关系")
    public ResultBean findRefDepartment(@PathVariable("userId") String userId) {
        try {
            ValidatorBean.checkNotNull(userId, "用户 id 不能为空");
            List<SysRefUserDepartment> list = sysUserInfoService.findRefUserDepartmentByUserId(Long.parseLong(userId));
            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultList(list);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 查询角色的所有角色权限关系
     *
     * @param model 用户明细模型
     * @return 处理结果
     */
    @PostMapping(value = "/query-ref-department")
    @ApiOperation(value = "查询角色的所有角色权限关系", notes = "查询角色的所有角色权限关系")
    public ResultBean queryRefDepartment(UserDetailModel model) {
        try {
            List<Long> idList = new ArrayList<>();

            List<SysRefUserDepartment> refs = sysUserInfoService.querySysRefUserDepartment(model);
            if (refs != null && refs.size() > 0) {
                for (SysRefUserDepartment ref : refs) {
                    idList.add(ref.getUserId());
                }
            }

            List<SysUserInfo> result = sysUserInfoService.querySysUserInfoList(model, idList);

            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultList(result);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 查询用户的所有角色权限关系
     *
     * @param userId 用户id
     * @return 处理结果
     */
    @GetMapping(value = "/find-ref-position/{userId}")
    @ApiOperation(value = "查询用户的所有角色权限关系", notes = "查询用户的所有角色权限关系")
    public ResultBean findRefPosition(@PathVariable("userId") String userId) {
        try {
            ValidatorBean.checkNotNull(userId, "用户 id 不能为空");
            List<SysRefUserPosition> list = sysUserInfoService.findSysRefUserPositionByUserId(Long.parseLong(userId));
            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultList(list);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 用户唯一信息校验
     *
     * @param userInfo 用户信息
     * @param isId     是否检查ID 存在
     */
    private void checkSysUserInfo(SysUserInfo userInfo, boolean isId) {

        try {
            //条件验证
            ValidatorBean.beginValid(userInfo)
                    .notNull("name", userInfo.getName())
                    .notNull("userEmpNo", userInfo.getUserEmpNo())
                    .checkNotZero("departmentId", userInfo.getDepartmentId())
//                    .checkNotZero("departmentIdList",userInfo.getDepartmentIdList())
                    .checkNotZero("positionId", userInfo.getPositionId());
            if (isId) {
                ValidatorBean.beginValid(userInfo)
                        .notNull("id", userInfo.getId());
            }

            SysDepartment department = departmentService.get(userInfo.getDepartmentId());
            if (department == null) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("主部门信息不存在。")
                        .build();
            }

            SysPosition position = positionService.get(userInfo.getPositionId());
            if (position == null) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("主岗位信息不存在。")
                        .build();
            }

            sysUserInfoService.checkSysUserInfoOnly(userInfo);

            userInfo.setDepartmentNameRdd(department.getName());
            userInfo.setPositionNameRdd(position.getName());
            userInfo.setDepartment(department);
            userInfo.setPosition(position);
        } catch (ImppBusiException busExcep) {
            LOGGER.error(busExcep.getErrorMsg() + "：{}", busExcep.getErrorDetail(), busExcep);
        } catch (Exception e) {
            LOGGER.error(LocaleUtils.getEnumLocaleResDesc(ImppExceptionEnum.SYSTEM_EXCEPTION,
                    ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription()) + "：{}", e.getMessage(), e);
        }
    }

    @GetMapping("/export")
    @ApiOperation(value = "导出用户信息", notes = "导出用户信息")
    public void exportSysUserInfo(SysUserInfo userInfo, Pager pager, HttpServletResponse response) throws Exception {
        File file = null;
        if(userInfo ==null ){
            userInfo = new SysUserInfo();
        }
        try {
            ListPager<SysUserInfo> result = sysUserInfoService.querySysUserInfo(userInfo, packUserInfo(userInfo), pager);
            List<SysUserInfo> list = result.getObjectList();
            Map<Long, SysUser> sysUserMap = sysUserService.list().stream()
                    .collect(Collectors.toMap(SysUser::getId, Function.identity()));
            List<UserExportModel> exportModelList = new ArrayList<>();


            UserExportModel userExportModel;
            List<SysRefUserRole> sysRefUserRoleList;
            StringBuilder roleName;
            for (SysUserInfo userInfoItem : list) {
                roleName = new StringBuilder();
                // 关联角色信息
                sysRefUserRoleList = sysUserService.findSysRefUserRoleByUserId(userInfoItem.getUserId());
                for (SysRefUserRole sysRefUserRole : sysRefUserRoleList) {
                    roleName.append(sysRefUserRole.getRoleNameRdd()).append(",");
                }
                if (roleName.length() != 0) {
                    roleName.setLength(roleName.length() - 1);
                }

                // 封装model
                userExportModel = new UserExportModel(sysUserMap.get(userInfoItem.getUserId()), userInfoItem);
                userExportModel.setRoleNameRdd(roleName.toString());

                exportModelList.add(userExportModel);
            }

            ExcelTool excelTool = new ExcelTool(entityManager, redisRes);
            String fileName = "user-info-" + System.currentTimeMillis() + ".xls";
            file = new File(fileName);
            file.createNewFile();
            File excle = excelTool.exportData(file, exportModelList, UserExportModel.class
                    , (LinkedHashMap<String, String>) ExcelTool.getColName(UserExportModel.class));

            response.setContentType("application/force-download"); // 设置强制下载不打开
            response.addHeader("Content-Disposition", "attachment;fileName=" + fileName); // 设置文件名
            response.addHeader("Content-type", FileContentTypeTool.getContentType(StringTool.getStringFileSuffix(fileName, true)));

            // 设置文件名
            try (BufferedInputStream bis = new BufferedInputStream(new DataInputStream(new FileInputStream(excle)))) {
                OutputStream os = response.getOutputStream();
                byte[] buffer = new byte[1024];
                int i = bis.read(buffer);
                while (i != -1) {
                    os.write(buffer, 0, i);
                    i = bis.read(buffer);
                }
            } catch (Exception e) {
                LOGGER.error("用户日志导出异常", e);
            }
        } finally {
            FileUtils.deleteQuietly(file);
        }
    }

    private ArrayList<Long> packUserInfo(SysUserInfo userInfo) {
        ArrayList<Long> idList = new ArrayList<>();
        if (userInfo.getOrganizeId() == null || userInfo.getOrganizeId() <= 0) {
            if (AuthUtil.getUserType().equals(CommonEnumUtil.USER_TYPE.ADMIN)) {
                userInfo.setOrganizeCode(null);
                userInfo.setOrganizeId(null);
            } else {
                userInfo.setOrganizeId(AuthUtil.getOrganize().getId());
                userInfo.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            }
        }

        List<Long> departmentIdList = memTreeService.findDepartmentIdListByOrganizeIdAndDepartmentId(
                userInfo.getOrganizeId(), userInfo.getDepartmentId()
        );
        if (departmentIdList != null && departmentIdList.size() > 0) {
            List<SysRefUserInfoDepartment> refList = personnelService.findSysRefUserInfoDepartmentByDepartmentIdList(departmentIdList);
            if (refList != null && refList.size() > 0) {
                refList.forEach(ref -> idList.add(ref.getUserId()));
            }
        }

        if (userInfo.getPositionId() != null && userInfo.getPositionId() > 0) {
            List<Long> refList = memTreeService.findSysPositionChildIdList(userInfo.getPositionId());
            if (refList != null && refList.size() > 0) {
                List<SysRefUserPosition> refUserPositionList = personnelService.findSysRefUserPositionByIdList(refList);
                if (refUserPositionList != null && refUserPositionList.size() > 0) {
                    refUserPositionList.forEach(ref -> idList.add(ref.getUserId()));
                }
            }
        }
        return idList;
    }

}
